#Requires -Version 4.0 
#Requires -Modules PKI 

<#
.SYNOPSIS 
   Replaces RDP certificate with certificate from TSPlus (https://www.tsplus.net) java keystore on local computers 
 
.DESCRIPTION 
   This function checks the thumbprint of the certificate in use by RDP against the the (Let's Encrypt) certificate from the
   TSPlus (https://www.tsplus.net) java keystore. If different it replaces the RDP certificate.
 
.USAGE 
   Replace-RDPCertificate-TSPlus 
  
.NOTES 
   Created by Yavuz Aydin for uHost  
   Last modified 2018/06/08  
   https://uhost.nl 
  
   Inspired by Tom Murphy's Install-RDPCert function
   https://gallery.technet.microsoft.com/scriptcenter/Remotely-install-RDP-8b9a82d3

   Inspired by Ryan Mangan's RDS 2012 Session Host Certificate Configuration script.  
   https://gallery.technet.microsoft.com/RDS-2012-Session-Host-fbb54ff9  
  
.LINK  
   https://uhost.nl
#> 

$jkskeystore = "C:\Progra~2\TSplus\Clients\webserver\cert.jks"
$storepass = "secret"
$securepass = $storepass | ConvertTo-SecureString -AsPlainText -Force  
$p12keystore = "$env:TEMP\cert.p12"
$javaversion = (java -version 2>&1 | select-string -pattern '.*java version "(.*)"'| % {$_.Matches.Groups[1].Value})
$javapath = "C:\Progra~2\Java\jre$javaversion\bin"
$keytool = "$javapath\keytool.exe"
$verbose = "Continue" # Continue for verbose output, SilentlyContinue for no output

$oldverbose = $VerbosePreference
$VerbosePreference = "$verbose"

If (Test-Path $p12keystore){
	Remove-Item $p12keystore
}
$keytool = (cmd /c "$keytool -importkeystore -srckeystore $jkskeystore -destkeystore $p12keystore -srcstoretype JKS -deststoretype PKCS12 -srcstorepass $storepass -deststorepass $storepass 2>&1")

try 
{ 
  $Thumbprint = (Get-PfxData -FilePath $p12keystore -Password $securepass).EndEntityCertificates.Thumbprint 
} 
catch [System.Exception]
{ 
  # Cannot get thumbprint from certificate, password is likely invalid 
  throw "${Env:ComputerName}: Access denied to certificate - ensure the password is correct" 
} # end try/catch 

try 
{ 
  $WMIObject = Get-WmiObject -Class "Win32_TSGeneralSetting" -Namespace root\cimv2\terminalservices -Filter "TerminalName='RDP-tcp'" -Authentication 6 
  if ($WMIObject){ 
    if ($WMIObject.SSLCertificateSHA1Hash -eq $Thumbprint){ 
      # Thumbprint already matches, no need to continue 
      Write-Verbose "${Env:ComputerName}: The certificate thumbprint already matches the new certificate" 
      Continue 
    } # end if 
  } else { 
    # WMI query did not return any results 
    Write-Warning "${Env:ComputerName}: Could not query WMI class Win32_TSGeneralSetting" 
    Continue 
  } # end if 
} 
catch 
{ 
  # Error connecting to WMI 
  Write-Warning "${Env:ComputerName}: Unable to connect to WMI`: $($Error[0])" 
  Continue 
} # end try/catch 

Write-Verbose "${Env:ComputerName}: Original thumbprint`: $($WMIObject.SSLCertificateSHA1Hash)" 

# Import certificate on remote machine 
Write-Verbose "${Env:ComputerName}: Importing certificate to LocalMachine store" 
$certRootStore = "LocalMachine"
$certStore = "My"
$pfx = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2Collection
$pfx.Import($p12keystore,$storepass,"Exportable,MachineKeySet,PersistKeySet")
$store = New-Object System.Security.Cryptography.X509Certificates.X509Store($certStore,$certRootStore)
$store.Open('ReadWrite')
$store.AddRange($pfx)
if ($?)
{
  "${Env:ComputerName}: Successfully added certificate."
}
else
{
  "${Env:ComputerName}: Failed to add certificate! $($Error[0].ToString() -replace '[\r\n]+', ' ')"
}
$store.Close()

Write-Verbose "${Env:ComputerName}: The certificate Thumbprint is: $Thumbprint"

wmic /namespace:\\root\CIMV2\TerminalServices PATH Win32_TSGeneralSetting Set SSLCertificateSHA1Hash="$Thumbprint" 2>&1

write-host "${Env:ComputerName}: The new certificate Thumbprint is: "$WMIObject.SSLCertificateSHA1Hash

Get-Service -Name 'Remote Desktop Services UserMode Port Redirector' | Stop-Service -Force -Verbose
Get-Service -Name 'TermService' | Stop-Service -Force -Verbose
Get-Service -Name 'TermService' | Start-Service -Verbose
Get-Service -Name 'Remote Desktop Services UserMode Port Redirector' | Start-Service -Verbose

$VerbosePreference = $oldverbose